function [f, varargout] = ellipsoid(x, y, a, b, c, str),

% ELLIPSOID Define an ellipsoid along the z-axis with apex at the origin. An ellipsoid 
% is given by,
% 
% z = c - sqrt(c^2 - (c*x/a).^2 - (c*y/b).^2)
% 
% where (a,b,c) are the semi-axes.  
% 
% ELLIPSOID(x, y, a, b, c, str). The pair (x, y) is the position in the (x, y) plane, 
% (a,b,c) are the semi-axes.  The optional parameter str is { 'z', 'dzdx', 'dzdy','normal' }.  
% If strf is omitted, the height ('z') will be returned. 
%
% [R, nArgList ] = ELLIPSOID('default').  Returns information for the RAYTRACE 
% toolbox, namely a default ellipsoid surface, along with the argument list.
%
% f = ELLIPSOID('inline').  Returns an inline version for the depth function ('z')   .
%
% Examples,
%   
%   nArgs = ellipsoid('default'); 
%   z = ellipsoid(1, 1.5, nArgs{:})
%   z = ellipsoid('inline'); z(1, 1.5, nArgs{:})
% 


% ELLIPSOID ellipsoid in the apex centered XYZ frame 
% 
% ELLIPSOID(x, y, a, b, c, strF). Return the ellipsoid 
% defined by (a,b,c) height/1st derivative at (x, y) 
%  f > 0 

if  ((nargin == 1)&(ischar(x))),
    switch(x),
        case { 'default' }
            f = { 8, 9, 10 };    
            varargout = { { 'a','b','c' } };
        case { 'inline' },
            f = 'c - sqrt(c^2 - (c*x/a).^2 - (c*y/b).^2)';
            f = inline(f,'x','y','a','b','c');
        otherwise,
            error('(ELLIPSOID) Unknown invocation parameter.');
    end 
   return
end


if (nargin == 5),
    f = sphere(c*x/a, c*y/b, c);
    return
end 

f = sphere(c*x/a, c*y/b, c, str);

switch(str)
 case 'z'
    return
 case 'dzdx'
    f = f*(c/a);
    return
 case 'dzdy'
    f = f*(c/b);
    return
 case 'normal'
    f(:,1) = f(:,1)*(c/a);
    f(:,2) = f(:,2)*(c/b);
    f(:,3) = -1;
    return;
 otherwise
     error('(ELLIPSOID) unknown aspect requested.');
end

return
