function M = bsp2dgetM(n, m, ktu, ktv, p1, p2, x, y)

% BSP2DGETM Get a 2D matrix for matrix computation of the b-spline height.
% If the B-Spline is represented by,  
%
%             N, M 
%            -------
%  Z(X, Y) = >>  C(I, J) B(I, J; PI, PJ)(X, Y) 
%            -------
%            I=0, J=0
%
% where 
%
%       B(I, J; P1, P2(X, Y) = B(I, P1)(X)*B(J, P2)(Y)
%
% then the M-matrix (denoted by U here!) is given by the following equation, 
%
%   Z(X, Y) = U*P
%
% where 
% 
%  U = [ B(0, 0) ... B(N, 0) ... B(0, M) ... B(N, M) ] 
%  P = [ C(0, 0) ... C(N, 0) ... C(0, M) ... C(N, M) ]^T
%
%
% M = BSP2DGETM(n, m, ktu, ktv, p1, p2, x, y).  Here (n, m) are maximum 
% indices, i.e., such that (n+1, m+1) are numbers of control points. The 
% knot-vectors are entered via (ktu, ktv), (p1, p2) is the degree of the 
% polynomial (where order = degree + 1, e.g. 2*x^3 is degree 3).  
%
% The vector P can be simply generated by using the (:) operator.  The 
% following example shows how to determine heights using this method,
%
% Example
%
%  % (n+1) control points, 4th order (degree p =3)
%  n = 6; p = 3; dm = [ -1 1 ]; 
%  x = linspace(-1, 1, 5); [x, y] = meshgrid(x, x);
%  kt = bsp1dnpknts(dm, n, p);  % knot seq. for u 
%  U = bsp2dgetM(n, m, kt, kt, p, p, x(:), y(:));
%  z = U*P(:);

M = []; x = x(:); y = y(:);

l=1;
for j = 0:m,
 for i = 0:n,
    M(:,l) = bsp1d(i, p1, ktu, x).*bsp1d(j, p2, ktv, y);
    l = l + 1;
 end
end

return