 function z = znkConicCoeffs(CV, CC, rp, nOrder, strf),

% ZNKCONICCOEFFS report analytic zernike coefficients for
% the conic with central radial curvature R and conic measure
% k using a 10^th order (66 term) Zernike polynomial.
%
% ZNKCONICCOEFFS(CV, CC, rp, nOrder).  Here CV is the conic curvature, 
% CC is the conic constant.  rp is radius of the aperture.  The 
% computation will be an integration based method.
% 
% ZNKCONICCOEFFS(CV, CC, rp, nOrder, strf).  Same as above except that 
% strf is specified as 'lsq' (least squares) or 'integral' for 
% integration based method.
% 
% EXAMPLE, Both methods should be consistent (10^th order).
% 
%  CV = 1/7.82; CC = -0.26; nApR = 4.5; nOrder = 10; 
%  C1 = znkConicCoeffs(CV, CC, nApR, nOrder);  % default ('integral') 
%  C2 = znkConicCoeffs(CV, CC, nApR, nOrder, 'lsq');   
%  [ C1(:) C2(:) ] 

% nOrder = 10;  % 10^th order Zernike
if (nargin == 0),
    R       = 7.82;   % radius
    K       = 0.82;
    CV      = 1/R;    % curvature 
    CC      = K-1;    % conic constant
    rp      = 3.0;    % 9 mm diam. aperture
else 
    R = 1/CV;
    K = 1+CC;
end 

if (nargin == 4),
    strf = 'integral';
end

switch(strf),
    case { 'lsq' },        
        r = linspace(0, 1, 33); r = r(2:end);
        t = linspace(0, 360, 33); t = degtorad(t(1:end-1));
        [r, t] = meshgrid(r, t); r = r(:); t = t(:);
        
        [x0, y0] = pol2cart(t, r);
        z0 = conic(x0, y0, CV, CC);
        
        nMaxTerm = znkOrder2Terms(nOrder)-1;
        [p(:,1), p(:,2)] = znkSgl2Dbl((0:1:nMaxTerm)');
        M  = znkBasisOSAVec(p, r/rp, t);
        z  = M\z0;
    case { 'integral' }
        n = znkOrder2Terms(nOrder);
        coeff = zeros(n,1);
        f   = [ '((CV*(rp*r).^2)./(1+sqrt(1-(CV)^2*(CC+1)*(rp*r).^2)))' ];
        tol  = 1.e-9;
        for k=0:2:nOrder,
            g  = [ '(' vectorize(znkTermStr(k, 0)) ')' ]; 
            h  = inline([ f '.*' g '.*r' ], 'r','CV','CC','rp');   
            j  = znkDbl2Sgl(k, 0);
            coeff(j+1) = 2*quad(h, 0, 1, tol, [], CV, CC, rp);
        end
        z = coeff;
    otherwise,
        error('(ZNKCONICCOEFFS) Unknown invocation parameter.');
end
return