function str_r = znkTermDvStr(n, m, strf, repl),

% ZNKTERMDVSTR Return string of the 1st derivative of Zernike basis (OSA)
%
% ZNKTERMDVSTR(n, m, <'rho','phi'>).  Here (n, m) is the (double-indexed) 
% term, and { 'dzdr', 'dzdt', 'rho', 'phi' } is the required 1st derivative.
% 
% Thibos, LN, Applegate, RA, Schwiegerling, JT, Webb, R. (2000).   
% Standards for reporting the optical Aberrations of Eyes.  
% In: Lakminarayan, V., ed. Trends in Optics and Photonics, Vision Science and Its Applications, 
% OSA Technical Digest Series, Washington, DC: Optical Society of America, Vol. 35:232-244.
% 
% Jim Schwiegerling, John E. Greivenkamp and Joseph M. Miller
% Representation of videokeratoscopic of height data with Zernike
% polynomials. Vol. 12, No. 10/ October 1995/J. Opt. Soc. Am. A


% 11/8 modified to avoid use of an 'abs(m)'.  Replaced with m instead.

% return and m=0, n=0 term
if ((m==0)&(n==0))
   str_r = '0';
   return
end


% (n = 1) case
if (n == 1),
 switch(strf),
        case { 'rho', 'dzdr' },
            if     (m == -1),
                str_r = '2*sin(phi)';
            elseif ( m == 1),
                str_r = '2*cos(phi)';
            else
                error('(ZNKTERMDVSTR) Unknown invocation.');
            end
        case { 'phi', 'dzdt' },
            if     (m == -1),
                str_r = '2*(r)*cos(phi)';
            elseif ( m == 1),
                str_r = '-2*(r)*sin(phi)';
            else
                error('(ZNKTERMDVSTR) Unknown invocation.');
            end
        otherwise,
         error('(ZNKTERMDVSTR) Unknown derivative.');
 end
 return   
end

% tangential deriv is zero when m == 0
if ((m ==0)&(strcmp(strf,'rho')|strcmp(strf,'dzdt')))
    str_r = '0';
    return
end


% more general cases ...
delta    = inline('n==0');
n_nm     = 2*(n+1)/(1+delta(m));
if (iswhole(sqrt(n_nm))),
    str_nm = sprintf('%d', sqrt(n_nm));
else
    str_nm   = sprintf('sqrt(%d)', n_nm);
end


str_r    = '';
for s=0:(n-abs(m))/2,
   
        % each zernike term 
        switch(strf),
            case { 'phi', 'dzdt' },
                
                p = n-2*s;
                nm  = (-1)^s*factorial(n-s); % *r.^(n-2*s);
                
                if (p > 1),
                    str_rt  = sprintf('r^%d', p);
                else
                    str_rt  = 'r';
                end    
                
            case { 'rho', 'dzdr' },
                                
                p = n-2*s-1;
                % if (n-2*s == 0), 
                %    nm      = 0;
                %    str_rt  = '';
                % if (else
                nm      = (-1)^s*factorial(n-s)*(n-2*s); %*r.^(n-2*s-1);    
                if     (p > 1),
                       str_rt  = sprintf('r^%d', p);
                elseif (p == 1),
                        str_rt  = 'r';
                else
                        str_rt  = ''; % sprintf('%d',nm);
                end 
                % end
            otherwise,
                error('(ZNKTERMDVSTR) Unknown derivative.');
        end
                    
        l = 0.5*(n+abs(m))-s;
        if (isequal(floor(l),l))
            dm      = factorial(s)*factorial(0.5*(n+abs(m))-s)*factorial(0.5*(n-abs(m))-s);
        else 
            warning([ 'fractional factorial! n = ' num2str(n) ', m = ' num2str(m) ]); 
        end
   
        if (nm ~= 0),
            
            if (nm/dm == 1),
                str_c = '';
            else 
                if (s == 0),
                    str_c  = sprintf('%d*', nm/dm);
                else
                    
                    if (p == 0),
                        str_c  = sprintf('%+d', nm/dm);
                    else
                        str_c  = sprintf('%+d*', nm/dm);
                    end
                end
            end
            
            str_r   = strcat(str_r, str_c, str_rt);                 
        end
        
end
    
    %if (n_nm == 1),
    %    str_r = strcat('(', str_r, ')');
    %else 
    str_r = strcat(str_nm, '*(', str_r, ')');
    %end 
    
    
    
    % angular part 
    switch(strf),
        case { 'rho', 'dzdr' },
            str_r = strcat(str_r, fixup(m, '', 'cos', 'sin'));
        case { 'phi', 'dzdt' },
            str_r = strcat(str_r, fixup(m, '0', 'sin', 'cos'));
                        
            if (abs(m) ~= 1),
                str = sprintf('%d', abs(m));
                str = strcat(asign(-m), str, '*');
            else
                str = asign(-m);              
            end 
            
            % if (abs(m) ~= 1),
            %    str = asign(m);
            % else
            %    str   = sprintf('%d*', -m);
            % end
            
            str_r = strcat(str, str_r);
   
        otherwise,
         error('(ZNKTERMDVSTR) Unknown derivative.');
    end
      
return


function y = iswhole(x),
    y = (floor(x) == x);
return


function y = asign(x),
    if (x<0),
        y = '-';
    else
        y = '';
    end
return

function y = signed(x),
    if (x ~= 0 ),
        y = sprintf('%+d', x);
    else
        y = '';
    end 
return



function str = fixup(m, str1, str2, str3),
         if  (m == 0),
             str = str1;
             return
         elseif (m == 1),
             str = strcat('*', str2, '(phi)');
         elseif (m > 1),
             str = sprintf('*%s(%d*phi)', str2, abs(m));
         elseif (m == -1),
             str = strcat('*', str3, '(phi)');
         elseif (m < -1),
             str = sprintf('*%s(%d*phi)', str3, abs(m));
         end
return     

