function [s, varargout] = prk(x, y, R1, R2, r1, r2, str),

% PRK Photorefractive Keratectomy
%
% PRK(x, y, R1, R2, r1, r2).  Here R1 is radius of curvature of the clear zone, 
% R2 is the radius of curvature of the outer zone, r1 is the start of the transition zone,
% and r2 is the end of the transition zone.  This region r1 <= r <= r2 will
% be smoothed using cubic spline interpolation. 
%
% PRK('default').  This will return default values for the PRK model.  
%
% NOTES : 
%
%  1. adpated from /videokeratography/nf_xyz_ablated.
%  2. checked against this method
%  3. this function must return matrix values for matrix input 
%     (element-wise return)

if  ((nargin == 1)&(ischar(x))),
    switch(x),
        case { 'default' }
             s = { 8, 6, 2.5, 3.5}; 
             varargout = { {'R1','R2','r1','r2'} };
        case { 'inline' },
            s = []; % no innline for this function yet;
        otherwise,
            error('(PRK) Unknown invocation parameter.');
    end
    return
end

% shift outer-zone so that the clear zone and outer zone meet at the 
% transition point a.  
a   = (r1+r2)/2;                                        % center of the transition zone;
d   = r2-r1;                                            % width of the transition zone
c2  = (sqrt(R1^2-a^2) - R1) - ( sqrt(R2^2 - a^2) - R2); % required shift

[t, r] = cart2pol(x, y);

if (nargin == 6),
    str = 'z';
end

switch(str)
 case 'z',

    i = (r <= r1);
    if (any(i(:))),
        s(i) = sphere(0, r(i), R1);
    end
    
    i = (r >= r2);
    if (any(i(:))),
        s(i) = sphere(0, r(i), R2)-c2;
    end
    
    i = ((r1 < r)&(r < r2));
    if (any(i(:))),
        s(i) = sphcub(r(i), R1, R2, r1, r2, c2, 'z');
    end
    s = reshape(s, size(r));
    return
 
 case 'dzdr',
    i = (r <= r1);
    if (any(i(:))),
        s(i) = sphere(0, r(i), R1, 'dzdr');
    end
    i = (r >= r2);
    if (any(i(:))),
        s(i) = sphere(0, r(i), R2, 'dzdr');
    end
    i = ((r1 < r)&(r < r2));
    if (any(i(:))),
        s(i) = sphcub(r(i), R1, R2, r1, r2, c2, 'dzdr');
    end
    
    s = reshape(s, size(r));
    return
     
 case 'dzdx',
     s    = prk(0, r, R1, R2, r1, r2, 'dzdr'); 
     s    = dpol2dcar(t, r, 0*s, s, 'dzdx');
    return
 case 'dzdy',
     s    = prk(0, r, R1, R2, r1, r2, 'dzdr'); 
     s    = dpol2dcar(t, r, 0*s, s, 'dzdy');
    return
 case 'normal'
    t = t(:); r = r(:);
    s0     = prk(0, r, R1, R2, r1, r2, 'dzdr'); 
    s(:,1) = dpol2dcar(t, r, 0*s0, s0, 'dzdx');
    s(:,2) = dpol2dcar(t, r, 0*s0, s0, 'dzdy');
    s(:,3) = -1;
    return;
 otherwise
     error('(PRK) Unknown aspect requested.');
end

return


function y = sphcub(r, R1, R2, r1, r2, c2, strf),

%
% RETURN POLYNOMIAL SURFACE BETWEEN TWO POINTS  
%

y(1) = sphere(0, r1, R1, 'z');
y(2) = sphere(0, r1, R1, 'dzdr');
y(3) = sphere(0, r1, R1, 'd2zdr2');
y(4) = sphere(0, r2, R2, 'z')-c2;
y(5) = sphere(0, r2, R2, 'dzdr');
y(6) = sphere(0, r2, R2, 'd2zdr2');

% y
% r1
% r2

M = [   r1.^5     r1.^4    r1.^3   r1.^2   r1.^1   1 ; ...  
      5*r1.^4   4*r1.^3  3*r1.^2 2*r1.^1     1     0 ; ... 
     20*r1.^3  12*r1.^2  6*r1.^1    2        0     0 ; ... 
        r2.^5     r2.^4    r2.^3   r2.^2   r2.^1   1 ; ...  
      5*r2.^4   4*r2.^3  3*r2.^2 2*r2.^1     1     0 ; ... 
     20*r2.^3  12*r2.^2  6*r2.^1    2        0     0 ]; 
a = inv(M)*y';       

% a

switch(strf),
    
    case { 'z' },
        y = a(1).*r.^5 + a(2).*r.^4 + a(3).*r.^3 + a(4).*r.^2 + a(5).*r + a(6);  
        % y
        
    case { 'dzdr' },
        y = 5*a(1)*r.^4 + 4*a(2)*r.^3 + 3*a(3)*r.^2 + 2*a(4)*r + a(5);
    otherwise,
        error('(PRK/SPHCUB) Unknown parameter.');
end

return